Package: ggvis
Title: Interactive Grammar of Graphics
Version: 0.4.10
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre")),
    person("Winston", "Chang", , "winston@posit.co", role = "aut"),
    person(, "Posit", role = "cph"),
    person(, "jQuery Foundation", role = "cph",
           comment = "jQuery library and jQuery UI library"),
    person(, "jQuery contributors", role = c("ctb", "cph"),
           comment = "jQuery library; authors listed in inst/www/lib/jquery/AUTHORS.txt"),
    person(, "jQuery UI contributors", role = c("ctb", "cph"),
           comment = "jQuery UI library; authors listed in inst/www/lib/jquery-ui/AUTHORS.txt"),
    person("Mike", "Bostock", role = c("ctb", "cph"),
           comment = "D3 library"),
    person(, "D3 contributors", role = "ctb",
           comment = "D3 library; authors listed at https://github.com/d3/d3/graphs/contributors"),
    person(, "Trifacta Inc.", role = "cph",
           comment = "Vega library"),
    person(, "Vega contributors", role = "ctb",
           comment = "Vega library; authors listed at https://github.com/trifacta/vega/graphs/contributors"),
    person("Sebastián", "Décima", role = c("ctb", "cph"),
           comment = "javascript-detect-element-resize library")
  )
Description: An implementation of an interactive grammar of graphics,
    taking the best parts of 'ggplot2', combining them with the reactive
    framework of 'shiny' and drawing web graphics using 'vega'.
License: GPL-2 | file LICENSE
URL: https://ggvis.rstudio.com/
Depends: R (>= 3.0)
Imports: assertthat, dplyr (>= 0.5.0), htmltools (>= 0.2.4), jsonlite
        (>= 0.9.11), magrittr, methods, rlang, shiny (>= 0.11.1)
Suggests: knitr (>= 1.6), lubridate, MASS, mgcv, rmarkdown, testthat
        (>= 0.8.1)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-02-10 14:25:27 UTC; hadleywickham
Author: Hadley Wickham [aut, cre],
  Winston Chang [aut],
  Posit [cph],
  jQuery Foundation [cph] (jQuery library and jQuery UI library),
  jQuery contributors [ctb, cph] (jQuery library; authors listed in
    inst/www/lib/jquery/AUTHORS.txt),
  jQuery UI contributors [ctb, cph] (jQuery UI library; authors listed in
    inst/www/lib/jquery-ui/AUTHORS.txt),
  Mike Bostock [ctb, cph] (D3 library),
  D3 contributors [ctb] (D3 library; authors listed at
    https://github.com/d3/d3/graphs/contributors),
  Trifacta Inc. [cph] (Vega library),
  Vega contributors [ctb] (Vega library; authors listed at
    https://github.com/trifacta/vega/graphs/contributors),
  Sebastián Décima [ctb, cph] (javascript-detect-element-resize library)
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2026-02-10 14:50:02 UTC
Built: R 4.6.0; ; 2026-04-28 22:50:01 UTC; unix
