% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/full_seq.R
\name{fullseq}
\alias{fullseq}
\title{Generate sequence of fixed size intervals covering range.}
\usage{
fullseq(range, size, ...)
}
\arguments{
\item{range}{range}

\item{size}{interval size}

\item{...}{other arguments passed on to methods}
}
\description{
Generate sequence of fixed size intervals covering range.
}
\keyword{internal}
