% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prop.R
\name{prop_domain}
\alias{prop_domain}
\title{Property domain.}
\usage{
prop_domain(x, data)
}
\arguments{
\item{x}{property to dispatch on}

\item{data}{name of data set}
}
\description{
Property domain.
}
