% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/broker.R
\name{is.broker}
\alias{is.broker}
\title{Determine if an object is a broker object}
\usage{
is.broker(x)
}
\arguments{
\item{x}{An object to test.}
}
\description{
Determine if an object is a broker object
}
