% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggvis.R
\name{is.ggvis}
\alias{is.ggvis}
\title{Is an object a ggvis object?}
\usage{
is.ggvis(x)
}
\arguments{
\item{x}{an object to test}
}
\description{
Is an object a ggvis object?
}
\keyword{internal}
