% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{default_options}
\alias{default_options}
\title{Default options}
\usage{
default_options()
}
\description{
This returns an object containing the default options for ggvis.
}
\keyword{internal}
